from __future__ import annotations

from .ara.AJGT import *
from .ara.HotelReviewSentimentClassification import *
from .ara.OnlineStoreReviewSentimentClassification import *
from .ara.RestaurantReviewSentimentClassification import *
from .ara.TweetEmotionClassification import *
from .ara.TweetSarcasmClassification import *
from .ben.BengaliDocumentClassification import *
from .ben.BengaliHateSpeechClassification import *
from .ben.BengaliSentimentAnalysis import *
from .bul.BulgarianStoreReviewSentimentClassfication import *
from .ces.CSFDCZMovieReviewSentimentClassification import *
from .ces.CzechProductReviewSentimentClassification import *
from .ces.CzechSoMeSentimentClassification import *
from .ces.CzechSubjectivityClassification import *
from .dan.AngryTweetsClassification import *
from .dan.DanishPoliticalCommentsClassification import *
from .dan.DKHateClassification import *
from .dan.LccSentimentClassification import *
from .deu.GermanPoliticiansTwitterSentimentClassification import *
from .deu.TenKGnadClassification import *
from .ell.GreekLegalCodeClassification import *
from .eng.AmazonPolarityClassification import *
from .eng.ArxivClassification import *
from .eng.Banking77Classification import *
from .eng.DBpediaClassification import *
from .eng.EmotionClassification import *
from .eng.FinancialPhrasebankClassification import *
from .eng.FrenkEnClassification import *
from .eng.ImdbClassification import *
from .eng.LegalBenchClassification import *
from .eng.NewsClassification import *
from .eng.PatentClassification import *
from .eng.PoemSentimentClassification import *
from .eng.SDSEyeProtectionClassification import *
from .eng.SDSGlovesClassification import *
from .eng.ToxicChatClassification import *
from .eng.ToxicConversationsClassification import *
from .eng.TweetSentimentExtractionClassification import *
from .eng.TweetTopicSingleClassification import *
from .eng.WikipediaBiolumNeurochemClassification import *
from .eng.WikipediaBioMetChemClassification import *
from .eng.WikipediaChemEngSpecialtiesClassification import *
from .eng.WikipediaChemFieldsClassification import *
from .eng.WikipediaChemistryTopicsClassification import *
from .eng.WikipediaCompChemSpectroscopyClassification import *
from .eng.WikipediaCryobiologySeparationClassification import *
from .eng.WikipediaCrystallographyAnalyticalClassification import *
from .eng.WikipediaGreenhouseEnantiopureClassification import *
from .eng.WikipediaIsotopesFissionClassification import *
from .eng.WikipediaLuminescenceClassification import *
from .eng.WikipediaOrganicInorganicClassification import *
from .eng.WikipediaSaltsSemiconductorsClassification import *
from .eng.WikipediaSolidStateColloidalClassification import *
from .eng.WikipediaTheoreticalAppliedClassification import *
from .eng.YahooAnswersTopicsClassification import *
from .eng.YelpReviewFullClassification import *
from .est.estonian_valence import *
from .fas.PersianFoodSentimentClassification import *
from .fil.FilipinoHateSpeechClassification import *
from .fil.FilipinoShopeeReviewsClassification import *
from .fin.FinToxicityClassification import *
from .fra.FrenchBookReviews import *
from .fra.MovieReviewSentimentClassification import *
from .guj.GujaratiNewsClassification import *
from .heb.HebrewSentimentAnalysis import *
from .hin.HindiDiscourseClassification import *
from .hin.SentimentAnalysisHindi import *
from .hrv.FrenkHrClassification import *
from .ind.IndonesianIdClickbaitClassification import *
from .ind.IndonesianMongabayConservationClassification import *
from .ita.ItaCaseholdClassification import *
from .ita.ItalianLinguistAcceptabilityClassification import *
from .jav.JavaneseIMDBClassification import *
from .jpn.WRIMEClassification import *
from .kan.KannadaNewsClassification import *
from .kor.KlueTC import *
from .kor.KorFin import *
from .kor.KorHateClassification import *
from .kor.KorSarcasmClassification import *
from .kur.KurdishSentimentClassification import *
from .mal.MalayalamNewsClassification import *
from .mar.MarathiNewsClassification import *
from .mkd.MacedonianTweetSentimentClassification import *
from .multilingual.AfriSentiClassification import *
from .multilingual.AfriSentiLangClassification import *
from .multilingual.AmazonCounterfactualClassification import *
from .multilingual.AmazonReviewsClassification import *
from .multilingual.CataloniaTweetClassification import *
from .multilingual.CyrillicTurkicLangClassification import *
from .multilingual.HinDialectClassification import *
from .multilingual.IndicLangClassification import *
from .multilingual.IndicNLPNewsClassification import *
from .multilingual.IndicSentimentClassification import *
from .multilingual.LanguageClassification import *
from .multilingual.MasakhaNEWSClassification import *
from .multilingual.MassiveIntentClassification import *
from .multilingual.MassiveScenarioClassification import *
from .multilingual.MTOPDomainClassification import *
from .multilingual.MTOPIntentClassification import *
from .multilingual.MultiHateClassification import *
from .multilingual.MultilingualSentimentClassification import *
from .multilingual.NaijaSenti import *
from .multilingual.NordicLangClassification import *
from .multilingual.NusaParagraphEmotionClassification import *
from .multilingual.NusaParagraphTopicClassification import *
from .multilingual.NusaXSenti import *
from .multilingual.ScalaClassification import *
from .multilingual.SIB200Classification import *
from .multilingual.SouthAfricanLangClassification import *
from .multilingual.SwissJudgementClassification import *
from .multilingual.TurkicClassification import *
from .multilingual.TweetSentimentClassification import *
from .mya.MyanmarNews import *
from .nep.NepaliNewsClassification import *
from .nld.DutchBookReviewSentimentClassification import *
from .nob.NoRecClassification import *
from .nob.NorwegianParliamentClassification import *
from .ory.OdiaNewsClassification import *
from .pan.PunjabiNewsClassification import *
from .pol.PolishClassification import *
from .por.HateSpeechPortugueseClassification import *
from .ron.Moroco import *
from .ron.RomanianReviewsSentiment import *
from .ron.RomanianSentimentClassification import *
from .rus.GeoreviewClassification import *
from .rus.HeadlineClassification import *
from .rus.InappropriatenessClassification import *
from .rus.KinopoiskClassification import *
from .rus.RuReviewsClassification import *
from .rus.RuSciBenchGRNTIClassification import *
from .rus.RuSciBenchOECDClassification import *
from .san.SanskritShlokasClassification import *
from .sin.SinhalaNewsClassification import *
from .sin.SinhalaNewsSourceClassification import *
from .slk.CSFDSKMovieReviewSentimentClassification import *
from .slk.SlovakHateSpeechClassification import *
from .slv.FrenkSlClassification import *
from .spa.SpanishNewsClassification import *
from .spa.SpanishSentimentClassification import *
from .ssw.SiswatiNewsClassification import *
from .svk.SlovakMovieReviewSentimentClassification import *
from .swa.SwahiliNewsClassification import *
from .swe.DalajClassification import *
from .swe.SwedishSentimentClassification import *
from .swe.SweRecClassification import *
from .tam.TamilNewsClassification import *
from .tel.TeluguAndhraJyotiNewsClassification import *
from .tha.WisesightSentimentClassification import *
from .tsn.TswanaNewsClassification import *
from .tur.TurkishMovieSentimentClassification import *
from .tur.TurkishProductSentimentClassification import *
from .ukr.UkrFormalityClassification import *
from .urd.UrduRomanSentimentClassification import *
from .vie.VieStudentFeedbackClassification import *
from .zho.CMTEBClassification import *
from .zho.YueOpenriceReviewClassification import (
    YueOpenriceReviewClassification,  # noqa: F401
)
from .zul.IsiZuluNewsClassification import *
